#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "logger.h"
#include "libcomment.h"

#define COMMENT_MODE_ERR 0
#define COMMENT_ADD "add"
#define COMMENT_MODE_ADD 1
#define COMMENT_VIEW "view"
#define COMMENT_MODE_VIEW 2

#define COMMENT_FIND "find"
#define COMMENT_MODE_FIND 3

#define COMMENT_FIND_ALL "findall"
#define COMMENT_MODE_FIND_ALL 4

#define COMMENT_FIND_HOME "findhome"
#define COMMENT_MODE_FIND_HOME 5

#define COMMENT_FIND_HERE "findhere"
#define COMMENT_MODE_FIND_HERE 6



struct COMMENT_globals {
	int verbose;
	int debug;
	int mode;
	int comment_start_index;
};

static char comment_version[]="Comment v0.99.1 - by Paul L Daniels - June 29, 2004 - http://www.pldaniels.com/comment";

static char comment_usage[]="Usage:\n"
"comment <add|find|findhome|findhere|findall> [-s] [--verbose] [--debug] <comment/findme, quoted as required>\n"
"\n"
"\tadd : Add a new comment to the present working directory .comment\n"
"\t\tdatabase and the ~/.comments database\n"
"\tfindhome: Look for 'findme' in the ~/.comments database\n"
"\tfindhere: Look in the current directory's .comment database\n"
"\tfind|findall: Look both in the current directory and the ~/.comments database\n"
"\t-s: Case sensitive search\n"
"\t--verbose: Turn on verbosity output\n"
"\t--debug: Turn on debugging output\n"
"\n";

/*-----------------------------------------------------------------\
 Function Name	: COMMENT_init
 Returns Type	: int
 	----Parameter List
	1. struct COMMENT_globals *glb , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int COMMENT_init( struct COMMENT_globals *glb )
{
	LOGGER_set_output_mode( _LOGGER_STDOUT );
	COMMENTLIB_set_debug(0);
	COMMENTLIB_set_verbose(0);
	COMMENTLIB_set_casesensitive(0);
	COMMENTLIB_set_find_mode( COMMENTLIB_FIND_CWD, 1 );
	COMMENTLIB_set_find_mode( COMMENTLIB_FIND_HOME, 0 );

	glb->verbose = 0;
	glb->debug = 0;
	glb->comment_start_index = 0;
	glb->mode = COMMENT_MODE_ERR;


	return 0;
}


/*-----------------------------------------------------------------\
 Function Name	: COMMENT_parse_parameters
 Returns Type	: int
 	----Parameter List
	1. struct COMMENT_globals *glb, 
	2.  int argc, 
	3.  char **argv , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int COMMENT_parse_parameters( struct COMMENT_globals *glb, int argc, char **argv )
{
	int i, tmpCWD = 0, tmpHOME = 0;
	int result = 0;
	char *p;

	if ( argc < 3 )
	{
		fprintf(stdout,"%s\n", comment_version);
		fprintf(stdout,"%s\n", comment_usage);
		exit(1);
	}

	for ( i = 1; i < argc; ++i ) {

	  p = argv[i];

	  /*
		* About command line argument position:
		*
		* Action MUST be first ( add|find|findhome|findhere|findall )
		*
		* Comment can be immediately following action, or alternately
		*     can come following verbose or debug; if not a single word
		*     it must be quoted
		*
		* --verbose or --debug can be second positionally, or last ONLY
		*     if the comment is a single word, or contained in quotes
		*
		* Legal:
		*  action "comment phrase" --debug --verbose
		*  action --debug "comment phrase" --verbose
		*  action --debug --verbose "comment phrase"
		*
		*/

	  if ( glb->mode ) {

		 if ( strncmp(p, "--debug", strlen("--debug")) == 0 ) {
			glb->debug = 1;
			COMMENTLIB_set_debug(1);
			LOGGER_log("%s:%d:COMMENT_parse_parameters: Debug mode active\n",FL);
		 }
		 else if ( strncmp(p, "--verbose", strlen("--verbose")) == 0 ) {
			glb->verbose = 1;
			COMMENTLIB_set_verbose(1);
			LOGGER_log("%s:%d:COMMENT_parse_parameters: Verbose mode active\n",FL);
		 }
		 else if ( strcmp(p, "-s") == 0) {
			 COMMENTLIB_set_casesensitive(1);
		 }
		 else {
			if ( glb->comment_start_index == 0 ) {
			  glb->comment_start_index = i;
			}
			else {
			  LOGGER_log("%s:%d:COMMENT_parse_parameters: comment already parsed: \"%s\"\n",FL,argv[glb->comment_start_index]);
			  exit(1);
			}
		 }

	  }
	  else {

		 if (strncmp(p, COMMENT_ADD, strlen(argv[i]))==0) { glb->mode = COMMENT_MODE_ADD; }
		 else if (strncmp(p, COMMENT_VIEW, strlen(argv[i]))==0) { glb->mode = COMMENT_MODE_VIEW; }

		 else if (strncmp(p, COMMENT_FIND, strlen(argv[i]))==0) { 
			glb->mode = COMMENT_MODE_FIND; 
			tmpCWD=1;
			tmpHOME=1;
		 }

		 else if (strncmp(p, COMMENT_FIND_HERE, strlen(argv[i]))==0) {
			glb->mode = COMMENT_MODE_FIND;
			tmpCWD=1;
			tmpHOME=0;
		 }
		 else if (strncmp(p, COMMENT_FIND_HOME, strlen(argv[i]))==0) {
			glb->mode = COMMENT_MODE_FIND;
			tmpCWD=0;
			tmpHOME=1;
		 }
		 else if (strncmp(p, COMMENT_FIND_ALL, strlen(argv[i]))==0) {
			glb->mode = COMMENT_MODE_FIND;
			tmpCWD=1;
			tmpHOME=1;
		 }
		 else {
			LOGGER_log("%s:%d:COMMENT_parse_parameters: Cannot understand the mode '%s'\n", FL, p );
			exit(1);
		 }
	  }
	}

	// Delayed call of set_find for --debug flag to be set
	COMMENTLIB_set_find_mode( COMMENTLIB_FIND_HOME, tmpHOME );
	COMMENTLIB_set_find_mode( COMMENTLIB_FIND_CWD, tmpCWD );

	return result;
}





/*-----------------------------------------------------------------\
 Function Name	: main
 Returns Type	: int
 	----Parameter List
	1. int argc, 
	2.  char **argv , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int main( int argc, char **argv )
{
	struct COMMENT_globals glb;

	COMMENT_init( &glb );


	COMMENT_parse_parameters( &glb, argc, argv );

	// If insufficient parameters are given to us then 
	//	we had better report to stdout the correct usage
	//
	//	Previously we could forget to leave a 'comment' and
	//	this would cause a nice little segfault.

	if ( argv[ glb.comment_start_index ] == NULL )
	{
		fprintf(stdout,"%s\n", comment_version);
		fprintf(stdout,"Insufficient parameters supplied for comment\n");
		fprintf(stdout,"%s\n", comment_usage);
		exit(1);
	}

	COMMENTLIB_config_load();

	switch (glb.mode)
	{
		case COMMENT_MODE_ADD:
			COMMENTLIB_add_comment( argv[ glb.comment_start_index ] );
			break;
		case COMMENT_MODE_VIEW:
			LOGGER_log("%s:%d:View mode not yet implemented - 25-Feb-2003",FL);
			break;
		case COMMENT_MODE_FIND:
			COMMENTLIB_find_comment( argv[ glb.comment_start_index ] );
			break;

		default:
			LOGGER_log("%s:%d:Unknown comment mode '%d'",FL, glb.mode );
	}

	return 0;
}
