/*------------------------------------------------------------------------
Module:        /extra/development/xamime/xamime_working/comment/libcomment.c
Author:        Paul L Daniels
Project:       Comment
State:         Alpha
Creation Date: 23-Feb-03
Description:   Comment is meant to be a blog/wiki style of system based on directory locations and user combined with current date/time.
This is more of a scattered wiki format, everyone who's allowed to access the file can either read/write comments based on their user/group rights.

------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <syslog.h>
#include <unistd.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <ctype.h>

#include "logger.h"
#include "libcomment.h"

#define FL __FILE__,__LINE__
#define COMMENTLIB_WORKING_DIR_FILE_NAME ".comment"
#define COMMENTLIB_HOME_DIR_FILE_NAME "comments"
#define COMMENTLIB_HOME_SUBDIR ".comments"

// Where to look for comment configuration files.
#define COMMENTLIB_CONFIG_PREFIX_COMMENT '#'
#define COMMENTLIB_CONFIG_SEPERATOR '='
#define COMMENTLIB_CONFIG_WORDWRAP "wordwrap"
#define COMMENTLIB_CONFIG_WRAPLENGTH "wraplength"
#define COMMENTLIB_CONFIG_WRAPLENGTH_MAXIMUM 1024
#define COMMENTLIB_CONFIG_WRAPLENGTH_DEFAULT 60

#define COMMENTLIB_DEBUG (commentlib_glb->debug)
#define COMMENTLIB_VERBOSE (commentlib_glb->verbose)

// All our configuration parameters are located in here
struct COMMENTLIB_conf {
	int wordwrap;
	int wraplength;
};

// All our global parameters which are passed around are here
struct COMMENTLIB_globals {
	int verbose;
	int debug;
	int find_cwd;
	int find_home;
	int case_sensitive;

	struct COMMENTLIB_conf conf;
};


static struct COMMENTLIB_globals C_glb, *commentlib_glb=&C_glb;

char *COMMENTLIB_CONFIG_FILES[]={	"/etc/comment.conf",	"~/.commentrc",	".commentrc",	NULL };

/*-----------------------------------------------------------------\
 Function Name	: COMMENTLIB_set_debug
 Returns Type	: int
 	----Parameter List
	1. int level , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int COMMENTLIB_set_debug( int level )
{
	commentlib_glb->debug = level;

	return commentlib_glb->debug;
}



/*------------------------------------------------------------------------
Procedure:     COMMENTLIB_set_verbose ID:1
Purpose:       Set the verbosity level of libcomment
Input:         int level : indicates what level of verbosity to use, 0 = none, > 0 = display
Output:        none
Errors:
------------------------------------------------------------------------*/
int COMMENTLIB_set_verbose( int level )
{
	commentlib_glb->verbose = level;

	return commentlib_glb->verbose;
}

/*-----------------------------------------------------------------\
 Function Name	: COMMENTLIB_set_casesensitive
 Returns Type	: int
 	----Parameter List
	1. int level , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int COMMENTLIB_set_casesensitive( int level )
{
	commentlib_glb->case_sensitive = level;

	return commentlib_glb->case_sensitive;
}




/*------------------------------------------------------------------------
Procedure:     COMMENTLIB_get_current_username ID:1
Purpose:       Determines the username of the person whom invoked the program
Input:         none
Output:        Returns a pointer to the username
Errors:        If the username is not found, a NULL pointer is returned.
------------------------------------------------------------------------*/
char *COMMENTLIB_get_current_user_name( void )
{
	int uid;
	struct passwd *userinfo;

	uid = getuid();
	userinfo = getpwuid(uid);

	if ( userinfo == NULL )
	{
		return NULL;
	}

	return userinfo->pw_name;
}


/*------------------------------------------------------------------------
Procedure:     COMMENTLIB_get_current_user_home ID:1
Purpose:       Retrieves the current users's home directory as detailed in the /etc/passwd file
Input:         none
Output:        returns a pointer to the home directory ( string )
Errors:
------------------------------------------------------------------------*/
char *COMMENTLIB_get_current_user_home( void )
{
	int uid;
	struct passwd *userinfo;

	uid = getuid();
	userinfo = getpwuid(uid);

	if ( userinfo == NULL )
	{
		return NULL;
	}

	return userinfo->pw_dir;
}



/*-----------------------------------------------------------------\
 Function Name	: COMMENTLIB_config_pair_parse
 Returns Type	: int
 	----Parameter List
	1. struct COMMENTLIB_globals *glb, 
	2.  int line, 
	3.  char *name, 
	4.  char *value , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int COMMENTLIB_config_pair_parse( struct COMMENTLIB_globals *glb, int line, char *name, char *value )
{
	if (COMMENTLIB_DEBUG) LOGGER_log("%s:%d:COMMENTLIB_config_pair_parse: Parsing '%s' and '%s'", FL, name, value);
	if (strstr( name, COMMENTLIB_CONFIG_WORDWRAP ))
	{
		switch (*value) {
		case 'y':
		case 'Y':
			glb->conf.wordwrap=1;
			break;

		case 'n':
		case 'N':
			glb->conf.wordwrap=0;
			break;

		default:
			LOGGER_log("COMMENTLIB_config_pair_parse: Unknown value for %s config variable. Please use '=y' or '=n'", COMMENTLIB_CONFIG_WORDWRAP);
			glb->conf.wordwrap=0;
		}
	}
	else if (strstr( name, COMMENTLIB_CONFIG_WRAPLENGTH )) {
		glb->conf.wraplength = atoi(value);
		if (( glb->conf.wraplength < 1 ) || (glb->conf.wraplength > COMMENTLIB_CONFIG_WRAPLENGTH_MAXIMUM ))
		{
			glb->conf.wraplength = COMMENTLIB_CONFIG_WRAPLENGTH_DEFAULT;
		}
	} else {
		LOGGER_log("COMMENTLIB_config_pair_parse: WARNING: Unknown configuration parameter on line %d: '%s'", line, name );
	}

	if (COMMENTLIB_DEBUG) LOGGER_log("%s:%d:COMMENTLIB_config_pair_parse: Done.", FL);

	return 0;

}


/*-----------------------------------------------------------------\
 Function Name	: COMMENTLIB_config_file_parse
 Returns Type	: int
 	----Parameter List
	1. struct COMMENTLIB_globals *glb, 
	2.  FILE *f , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int COMMENTLIB_config_file_parse( struct COMMENTLIB_globals *glb, FILE *f )
{
	int line = 0;
	char buffer[1024];
	char *name, *value;
	char *breakpoint;

	while (fgets( buffer, sizeof(buffer), f))
	{
		line++;
		if ( buffer[0] != COMMENTLIB_CONFIG_PREFIX_COMMENT )
		{
			if (strlen(buffer) > 3)
			{
				breakpoint = strchr(buffer, COMMENTLIB_CONFIG_SEPERATOR );
				if (breakpoint != NULL)
				{
					name = buffer;
					value = breakpoint +1;
					*breakpoint = '\0';
					COMMENTLIB_config_pair_parse( glb, line, name, value );
				}
			}
		}
	}

	return 0;
}

/*-----------------------------------------------------------------\
 Function Name	: COMMENTLIB_config_load_single
 Returns Type	: int
 	----Parameter List
	1. struct COMMENTLIB_globals *glb, 
	2.  char *filename , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int COMMENTLIB_config_load_single( struct COMMENTLIB_globals *glb, char *filename )
{
	int stat_result;
	char full_filename[1024];
	struct stat st;
	FILE *f;

	if (COMMENTLIB_DEBUG) LOGGER_log("%s:%d:COMMENTLIB_config_load_single: Checking config file '%s'", FL, filename );

	if (strncmp(filename, "~/", strlen("~/")) == 0)
	{
		snprintf( full_filename, sizeof(full_filename), "%s/%s", COMMENTLIB_get_current_user_home(), filename +strlen("~/") );
	}
	else {
		snprintf( full_filename, sizeof(full_filename), "%s", filename );
	}

	stat_result = stat( full_filename, &st );
	if (stat_result == 0)
	{
		f = fopen( full_filename, "r" );
		if (f == NULL)
		{
			LOGGER_log("COMMENTLIB_config_load_single: ERROR opening config file '%s' (%s)", full_filename, strerror(errno));
			return -1;
		}

		if (COMMENTLIB_DEBUG) LOGGER_log("%s:%d:COMMENTLIB_config_load_single: Parsing file '%s'", FL, full_filename );

		COMMENTLIB_config_file_parse( glb, f );

		fclose(f);
	}

	return 0;


	return 0;
}



/*-----------------------------------------------------------------\
 Function Name	: COMMENTLIB_config_apply
 Returns Type	: int
 	----Parameter List
	1. struct COMMENTLIB_globals *glb , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int COMMENTLIB_config_apply( struct COMMENTLIB_globals *glb )
{

//	LOGGER_log("%s:%d: wordwrap = %d, wraplength = %d", FL, glb->conf.wordwrap, glb->conf.wraplength );
//	LOGGER_set_wrap( glb->conf.wordwrap );
//	LOGGER_set_wraplength( glb->conf.wraplength );

	return 0;
}


/*-----------------------------------------------------------------\
 Function Name	: COMMENTLIB_config_load
 Returns Type	: int
 	----Parameter List
	1. void , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int COMMENTLIB_config_load( void )
{
	char **commentfile;

	commentfile = COMMENTLIB_CONFIG_FILES;

	while ( *commentfile )
	{
		COMMENTLIB_config_load_single( commentlib_glb, *commentfile );
		commentfile++;
	}

	COMMENTLIB_config_apply( commentlib_glb );

	return 0;
}



/*------------------------------------------------------------------------
Procedure:     COMMENTLIB_set_find_mode ID:1
Purpose:       Set the methods and locations to search for and find comments
Input:         int find_mode: Which mode to set [ _HOME or _CWD ]
int level: What level is the mode set to 0 == don't use, > 0 == use
Output:
Errors:
------------------------------------------------------------------------*/
int COMMENTLIB_set_find_mode( int find_mode, int level )
{
	switch (find_mode) {
	case COMMENTLIB_FIND_HOME:
		if (COMMENTLIB_DEBUG) LOGGER_log("%s:%d:COMMENTLIB_set_find: Home search set to %d", FL, level );
		commentlib_glb->find_home = level;
		break;
	case COMMENTLIB_FIND_CWD:
		if (COMMENTLIB_DEBUG) LOGGER_log("%s:%d:COMMENTLIB_set_find: CWD search set to %d", FL, level );
		commentlib_glb->find_cwd = level;
		break;
	default:
		LOGGER_log("%s:%d:COMMENTLIB_set_find: Unknown find mode '%d'", find_mode );
	}

	return 0;
}



/*------------------------------------------------------------------------
Procedure:     COMMENTLIB_file_comment_write ID:1
Purpose:       Writes a comment to an output file, encoding any special characters on the fly.
Input:         FILE *f: output data stream
char *data: pointer to the start of the data
int data_len: length of data sequence
Output:
Errors:
------------------------------------------------------------------------*/
int COMMENTLIB_file_comment_write( struct COMMENTLIB_globals *glb, FILE *f, char *data, int data_len )
{
	char *p;
	char *next_space;
	int last_char_was_linebreak = 1;
	int header_is_written = 0;
	int line_size;

	// All new comments start with a leading # character
	fwrite( "#", 1, strlen("#"), f );


	// Because we need to test for \n# sequences, we are forced to move
	//		through the data one character at a time, fortunately, because
	//		we're dealing with relatively small amounts of data this does
	//		not impose a major performance overhead.
	//
	// If we detect a '#' char preceeded by a \n char, we write a single
	//		backslash before the '#' is written.
	//
	// If we detect a \n or \r, then we mark a flag to indicate that our
	//		line was just terminated.

	p = data;
	line_size = 0;
	while ((*p != '\0')&&(data_len > 0))
	{

		if (( header_is_written == 1 )&&( glb->conf.wordwrap > 0 ))
		{
			if (isspace(*p))
			{
				next_space = strpbrk( (p+1), "\t\r\n\v " );
				if (next_space != NULL)
				{
					if ((line_size +(next_space -p)) >= glb->conf.wraplength)
					{
						fwrite( "\n", 1, 1, f );
						line_size = 0;
						last_char_was_linebreak = 1;
					}
				}
			}

			if ( line_size >= glb->conf.wraplength )
			{
				fwrite( "\n", 1, 1, f );
				line_size = 0;
				last_char_was_linebreak = 1;
			}
		}


		if ((*p == '#')&&(last_char_was_linebreak != 0))
		{
			fwrite( "\\", 1, sizeof("\\"), f );
		}

		if ((*p == '\n')||(*p == '\r')){
			last_char_was_linebreak = 1;
			header_is_written = 1;
		}
		else last_char_was_linebreak = 0;

		fwrite( p, 1, sizeof(char), f );

		p++;
		data_len--;
		line_size++;
	}

	return 0;
}





/*------------------------------------------------------------------------
Procedure:     COMMENTLIB_file_prefix ID:1
Purpose:       Prefix a comment to an existing file
Input:         char *filename: path of the file we wish to prefix to
char *comment: the data we wish to prefix to the file
Output:
Errors:
------------------------------------------------------------------------*/
int COMMENTLIB_file_prefix( struct COMMENTLIB_globals *glb, char *filename, char *comment )
{
	FILE *tmp_file;
	FILE *original_file;
	char buffer[4096];
	int unlink_result, read_size, write_size, rename_result;

	if (COMMENTLIB_DEBUG) LOGGER_log("%s:%d:COMMENTLIB_file_prefix: Writing to '%s' comments '%s'", FL, filename, comment );
	original_file = fopen( filename, "r" );
	if ( original_file == NULL )
	{
		if (COMMENTLIB_VERBOSE) LOGGER_log("%s:%d:COMMENTLIB_file_prefix: WARNING: Unable to open file '%s'(%s)", FL, filename, strerror(errno));
		// Original file doesn't exist, usually this simply means that
		//		we need to create one.
						}

	snprintf( buffer, sizeof(buffer), "%s.tmp", filename );
	tmp_file = fopen( buffer, "w" );
	if ( tmp_file == NULL )
	{
		LOGGER_log("%s:%d:COMMENTLIB_file_prefix: ERROR: Unable to open temporary file '%s'(%s)", FL, buffer, strerror(errno));
		// If we can't create the tmp file, then throw a fit
		return -1;
	}

	// Write the new comment and encode the output

	COMMENTLIB_file_comment_write( glb, tmp_file, comment, strlen(comment) );

	// If an existing file does exist ( original ), then we need to now 'append'
	//		it to the end of our new file - this is how we perform the 'prefixing'

	if ( original_file != NULL )
	{
		do {
			read_size = fread( buffer, 1, sizeof(buffer), original_file );
			if ( read_size >  0 )
			{
				write_size = fwrite( buffer, 1, read_size, tmp_file );
				if ( write_size != read_size )
				{
					LOGGER_log("COMMENT_file_prefix: WARNING, attempted to write %d bytes to %s.tmp but only wrote %d", read_size, filename, write_size );
				}
			}
		}
		while ( read_size >  0 );
		fclose(original_file);
	}

	fclose(tmp_file);

	// Move the existing original file to a safe place
	//		ie, foobar.old

	snprintf( buffer, sizeof(buffer), "%s.old", filename );
	rename_result = rename( filename, buffer );
	if ( rename_result > 0 )
	{
		LOGGER_log("COMMENT_prefix_file: ERROR renaming %s to %s (%s)", filename, buffer, strerror(errno));
		return -1;
	}

	// Move the newly created temporary file to the original
	//		filename, ie, foo.tmp -> foo

	snprintf( buffer, sizeof(buffer), "%s.tmp", filename );
	rename_result = rename( buffer, filename );
	if ( rename_result > 0 )
	{
		// If the rename fails for what ever reason, then revert/roll-back to the original file.

		LOGGER_log("COMMENT_prefix_file: ERROR renaming %s to %s (%s), reverting back to original file", buffer, filename, strerror(errno));
		snprintf( buffer, sizeof(buffer),"%s.old", filename );
		rename_result = rename( buffer, filename );
		if ( rename_result > 0 )
		{
			// If things -still- fail then there's very little hope of success here, so just
			//		leave a nice log message and hope that the administrator/user read it and
			//		checks things out ( perhaps permissions issues? broken filesystem? )

			LOGGER_log("COMMENT_prefix_file: ERROR renaming %s to %s (%s) while attempting to recover from a previously failed file rename.", buffer, filename, strerror(errno));
			return -1;
		}

		return -1;
	}

	// Dispose of the old file.

	snprintf( buffer, sizeof(buffer), "%s.old", filename );
	unlink_result = unlink( buffer );
	if ( unlink_result > 0 )
	{
		LOGGER_log("COMMENT_prefix_file: ERROR removing %s (%s)", buffer, strerror(errno));
		return -1;
	}

	if (COMMENTLIB_DEBUG) LOGGER_log("%s:%d:COMMENTLIB_file_prefix: Done", FL );
	return 0;
}


/*------------------------------------------------------------------------
Procedure:     COMMENT_check_home_path ID:1
Purpose:       Check to see if the home path ~/.comments exists, if not, take corrective action
Input:         char *path: string consisting of the path representing the home directory
Output:
Errors:
------------------------------------------------------------------------*/
int COMMENT_check_home_path( char *path )
{
	struct stat st;
	int result;

	// Check to see if the 'comment' home directory exists in the users's home,
	//		if it doesn't, then attempt to create it.  Should the creation fail we
	//		just leave an error message. We let the calling program decide what it'll
	//		do about this matter.

	result = stat( path, &st );
	if ( result != 0 )
	{
		result = mkdir( path, S_IRWXU );
		if ( result != 0 )
		{
			LOGGER_log( "%s:%d:COMMENTLIB_check_home_path: ERROR: Cannot create directory '%s'", FL, path );
			return -1;
		}
	}

	return 0;
}





/*------------------------------------------------------------------------
Procedure:     COMMENTLIB_add_comment_details ID:1
Purpose:       Composes and writes a comment based on the composition of the supplied parameters, this call invokes
a comment write attempt for every required comment file service ( CWD, Home, DB etc )
Input:         char *working_dir: The current working directory in which comment was invoked
char *home_dir: The home directory of the user who invoked comment
char *date_string: The current date/time as desired to be represented in the comment files
char *user_name: The login name of the user invoking comment
char *comment: Actual comment which is to be written to the files/DB

Output:
Errors:
------------------------------------------------------------------------*/
int COMMENTLIB_add_comment_details( struct COMMENTLIB_globals *glb, char *working_dir, char *home_dir, char *date_string, char *user_name, char *comment )
{
	char user_unknown[]="Unknown User";
	char *data_string;
	char file_path[1024];
	int required_data_size;
	int result;

	// If we can't determine the username [ for what ever reason ], we'll simply set it
	//		to unknown user, this way we aren't left with a 'blank' space in the comment
	//		headers.

	if ( user_name == NULL )
	{
		user_name = user_unknown;
	}

	// Allocate enough memory for the creation of a outgoing comment string.

	required_data_size = strlen( comment ) +strlen( date_string ) +strlen( working_dir ) +strlen( user_name ) +1024;
	data_string = malloc( required_data_size *sizeof(char));
	if ( data_string == NULL )
	{
		LOGGER_log("COMMENTLIB_add_comment_details: ERROR: Unable to allocate %d bytes of memory", (required_data_size *sizeof(char)));
		return -1;
	}

	// Compose the working directory .comment file string
	// 	and write the COMMENT to the working dir .comment file

	if ( working_dir != NULL )
	{
		snprintf( data_string, required_data_size, "%s - %s - %s\n%s\n\n", date_string, user_name,  working_dir, comment );
		snprintf( file_path, sizeof(file_path), "%s/%s", working_dir, COMMENTLIB_WORKING_DIR_FILE_NAME );
		COMMENTLIB_file_prefix( glb, file_path, data_string );
	}

	// Compose the Home directory comment file string, check the
	//		existance ( and create if needed ) of the home directory
	//		.comments subdir, then write the comment ( if all went well )
	//
	//	If we're unable to write to the home directory comment file, then
	//		we will leave a small log/error report but continue on without
	//		an exit ( because the comment write may succeed for other files
	//		or DB write requests )

	if ( home_dir != NULL )
	{
		snprintf( data_string, required_data_size, "%s/%s", home_dir, COMMENTLIB_HOME_SUBDIR );
		result = COMMENT_check_home_path( data_string );
		if ( result == 0 )
		{
			snprintf( data_string, required_data_size, "%s - %s - %s\n%s\n\n", date_string, user_name, working_dir, comment );
			snprintf( file_path, sizeof(file_path), "%s/%s/%s", home_dir, COMMENTLIB_HOME_SUBDIR, COMMENTLIB_HOME_DIR_FILE_NAME );
			COMMENTLIB_file_prefix( glb, file_path, data_string );
		}
		else {
			LOGGER_log("COMMENTLIB_add_comment_details: WARNING: Unable to write comment to the home directory");
		}
	}

	// Don't forget to free up any resources we allocated to perform this
	//		process.  While not 'critical', it's not good habit to forget to
	//		free resources.

	if (data_string) free(data_string);

	return 0;

}



/*------------------------------------------------------------------------
Procedure:     COMMENTLIB_add_comment ID:1
Purpose:       Adds a comment to the comment data files
Input:         char *comment: nul terminated character string consisting of the comment required to be saved
Output:        none
Errors:
------------------------------------------------------------------------*/
int COMMENTLIB_add_comment( char *comment )
{

	char *p;
	char user_name[128];
	char home_dir[128];
	char working_dir[256];
	char date_string[128];
	time_t t;

	// Get the user name ( login username ) and the
	//	current home directory of the user logged in.
	//
	// We snprintf these strings because we cannot gaurantee
	//		that the pointer will be preserved between calls

	snprintf(user_name, sizeof(user_name),"%s", COMMENTLIB_get_current_user_name());
	snprintf(home_dir, sizeof(home_dir),"%s", COMMENTLIB_get_current_user_home());


	// Compose the current date string.  Make sure we crop off
	//		the trailing \n which is created when using ctime()
	//		else this will break the header output line in the comment
	//		file.

	t = time(NULL);
	snprintf( date_string, sizeof(date_string), "%s", ctime(&t));
	p = strrchr( date_string, '\n' );
	if (p != NULL ) *p = '\0';

	// Get our current-working-directory, or basically where 'comment'
	//		was invoked.

	getcwd( working_dir, sizeof(working_dir));

	COMMENTLIB_add_comment_details( commentlib_glb, working_dir, home_dir, date_string, user_name, comment );

	return 0;
}


/*-----------------------------------------------------------------\
 Function Name	: COMMENTLIB_strlower
 Returns Type	: int
 	----Parameter List
	1. char *lowerme , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int COMMENTLIB_strlower( char *lowerme )
{
	while ((lowerme != NULL)&&(*lowerme != '\0'))
	{
		*lowerme = tolower(*lowerme);
		lowerme++;
	}
	return 0;
}

/*------------------------------------------------------------------------
Procedure:     COMMENTLIB_comment_buffer_search ID:1
Purpose:       Performs the fundamental search within a haystack
Input:         char *haystack: Entire data set in which we are searching
char *needle: What we are attempting to locate within the haystack
Output:        Returns 0 == No search success
Returns 1 == needle was found
Errors:
------------------------------------------------------------------------*/
int COMMENTLIB_comment_buffer_search( char *haystack, char *needle )
{
	int result = 0;
	char *p;
	char *n = needle, *h = haystack;
	int free_strings = 0;

	if ( haystack[0] != '\0' )
	{

		if (commentlib_glb->case_sensitive == 0)
		{
			// If we're going to search case-insensitive, then
			//		duplicate our strings and then mutate them to
			//		all lowercase before searching.
			//
			// Remeber to free the allocated strings when done.

			n = strdup(needle);
			h = strdup(haystack);

			if ((n != NULL) &&( h != NULL ))
			{
				COMMENTLIB_strlower(h);
				COMMENTLIB_strlower(n);
				free_strings = 1;
			} else {
				LOGGER_log("%s:%d: There was a problem trying to duplicate the needle and haystack strings for the search, exiting",FL);
				exit(1);
			}
		}

		if (COMMENTLIB_DEBUG) LOGGER_log("%s:%d:COMMENTLIB_comment_buffer_search: Searching in comment \"%s\"", FL, haystack  );
		p = strstr( h, n );
		if (p != NULL)
		{
			result = 1;
		}

		if (free_strings == 1)
		{ 
			free(n);
			free(h);
		}

	}
	return result;
}


/*------------------------------------------------------------------------
Procedure:     COMMENTLIB_search_report ID:1
Purpose:       Generate a report detailing the result of a single comment search hit.
Input:         char *header: Header information which is obtained at the start of every comment
char *comment: Actual comment block in which search 'hit'
int comment_count: Which comment ( 1-based index ) this is ( relative to the entire file )
int line_count: Which line the comment was located on within the file
Output:
Errors:
------------------------------------------------------------------------*/
int COMMENTLIB_search_report( char *header, char *comment, int comment_count, int line_count )
{

	LOGGER_log( "\t%d:%d:%s%s", line_count, comment_count, header, comment );

	return 0;
}



/*------------------------------------------------------------------------
Procedure:     COMMENTLIB_find_comment_infile ID:1
Purpose:       Search for a search string in given filename
Input:         char *filename: string containing path of file to search in
char *searchfor: string containing search details
Output:
Errors:
------------------------------------------------------------------------*/
int COMMENTLIB_find_comment_infile( char *filename, char *searchfor )
{
	FILE *f;
	char *fgets_result;
	struct stat st;
	int result;
	int comment_buffer_len = 0;
	int line_buffer_len = 0;
	int comment_count = 0;
	int line_count = 0;
	char header_line[1024];
	char comment_buffer[10240];
	char buffer[1024];

	if (COMMENTLIB_DEBUG) LOGGER_log("%s:%d:COMMENTLIB_find_comment_infile: Filename = %s,  Search for '%s'", FL, filename, searchfor );

	// Test for the presence of the file we are required to search in.
	//		There is little point attempting to search in a non-existant file
	//	If the file doesn't exist, then we should leave a little note and exit
	//		moderately quietly ( so as not to break any following search attempts
	//		on other files.

	result = stat( filename, &st );
	if (( result == 0 )&&(S_ISREG(st.st_mode)))
	{
		f = fopen( filename, "r" );
		if ( f == NULL )
		{
			LOGGER_log("%s:%d:COMMENTLIB_find_comment_infile: Cannot open file '%s' for reading (%s)", FL, filename, strerror(errno));
			return -1;
		}

		do {

			fgets_result = fgets( buffer, sizeof(buffer), f );
			if ( fgets_result != NULL )
			{

				line_buffer_len = strlen( buffer );

				if (strrchr( buffer, '\n' ) != NULL) line_count++;

				if ( buffer[0] == '#' )
				{
					if (COMMENTLIB_comment_buffer_search( comment_buffer, searchfor ) > 0 )
					{

						COMMENTLIB_search_report( (header_line +1), comment_buffer, comment_count, line_count );
					}

					snprintf( header_line, sizeof(header_line), "%s", buffer );
					comment_buffer[0] = '\0';
					comment_count++;
					line_count++;
				}
				else {
					if ( (comment_buffer_len +line_buffer_len) < sizeof(comment_buffer) )
					{
						strcat( comment_buffer, buffer );
						comment_buffer_len += line_buffer_len;
					}
				}
			}

		}
		while (fgets_result != NULL);

		fclose(f);

		if (COMMENTLIB_comment_buffer_search( comment_buffer, searchfor ) > 0 )
		{
			COMMENTLIB_search_report( (header_line +1), comment_buffer, comment_count, line_count );
		}

	}

	return 0;
}





/*------------------------------------------------------------------------
Procedure:     COMMENTLIB_find_comment_details ID:1
Purpose:       Searches for comments amongst the various databases stored based on current user information
Input:         char *working_dir: Directory to search for .comment files
char *home_dir: User home directory ( search for ~/.comments dir )
char *user_name: User login name
char *search: Search string
Output:
Errors:
------------------------------------------------------------------------*/
int COMMENTLIB_find_comment_details( char *working_dir, char *home_dir, char *user_name, char *searchfor )
{

	char user_unknown[]="Unknown User";
	char file_path[1024];

	if (COMMENTLIB_DEBUG) LOGGER_log("%s:%d:COMMENTLIB_find_comment_details: Working_dir = %s, Home_dir = %s, User_name = %s, Search for '%s'", FL, working_dir, home_dir, user_name, searchfor );

	if ( user_name == NULL )
	{
		user_name = user_unknown;
	}

	// Compose the working directory .comment file string
	if (( working_dir != NULL )&&( commentlib_glb->find_cwd > 0 ))
	{
		snprintf( file_path, sizeof(file_path), "%s/%s", working_dir, COMMENTLIB_WORKING_DIR_FILE_NAME );
		COMMENTLIB_find_comment_infile( file_path, searchfor );
	}

	// Compose the Home directory comment file string
	if (( home_dir != NULL )&&( commentlib_glb->find_home > 0 ))
	{
		snprintf( file_path, sizeof(file_path), "%s/%s/%s", home_dir, COMMENTLIB_HOME_SUBDIR, COMMENTLIB_HOME_DIR_FILE_NAME );
		COMMENTLIB_find_comment_infile( file_path, searchfor );
	}


	return 0;
}




/*------------------------------------------------------------------------
Procedure:     COMMENTLIB_find_comment ID:1
Purpose:       Find and display comments matching the required search criterion
Input:         char *comment: Look for this comment
Output:
Errors:
------------------------------------------------------------------------*/
int COMMENTLIB_find_comment( char *comment )
{

	char user_name[128];
	char home_dir[128];
	char working_dir[256];

	if (COMMENTLIB_DEBUG) LOGGER_log("%s:%d:COMMENTLIB_find_comment: Searching for '%s'", FL, comment );

	// Get current user information

	snprintf(user_name, sizeof(user_name),"%s", COMMENTLIB_get_current_user_name());
	snprintf(home_dir, sizeof(home_dir),"%s", COMMENTLIB_get_current_user_home());
	getcwd( working_dir, sizeof(working_dir));

	// Try to locate our required comments ( This will also report the found comments
	//		so we don't have to worry about doing any reporting ourselves )
	//
	//	Later we may add a function 'callback' so that developers can add their own
	//		reporting functions.

	COMMENTLIB_find_comment_details( working_dir, home_dir, user_name, comment );

	return 0;
}


//--------------------END

